@@ -11,10 +11,12 @@ Interchange6::Schema::Result::Product
 use base 'Interchange6::Schema::Base::Attribute';
 
 use DateTime;
+use Encode;
 use Try::Tiny;
 
 use Interchange6::Schema::Candy -components =>
-  [qw(InflateColumn::DateTime TimeStamp)];
+  [qw(Helper::Row::OnColumnChange Helper::Row::SelfResultSet
+      InflateColumn::DateTime TimeStamp)];
 
 =head1 DESCRIPTION
 
@@ -88,12 +90,23 @@ column description => {
 
 Numeric value representing product cost. Default is 0.0.
 
+When C<price> is updated and product has related
+L<Interchange6::Schema::Result::PriceModifier/discount> then also update
+the related L<Interchange6::Schema::Result::PriceModifier/price>.
+This is done using the method C<update_price_modifiers>.
+
 =cut
 
 column price => {
-    data_type     => "numeric",
-    default_value => "0.0",
-    size          => [ 10, 2 ]
+    data_type          => "numeric",
+    default_value      => "0.0",
+    size               => [ 10, 2 ],
+    keep_storage_value => 1,
+};
+
+before_column_change price => {
+    method   => 'update_price_modifiers',
+    txn_wrap => 1,
 };
 
 =head2 uri
@@ -112,12 +125,14 @@ unique_column uri => {
 
 Numeric weight of the product.
 
+Is nullable.
+
 =cut
 
 column weight => {
-    data_type     => "numeric",
-    default_value => "0.0",
-    size          => [ 10, 2 ]
+    data_type   => "numeric",
+    size        => [ 10, 2 ],
+    is_nullable => 1,
 };
 
 =head2 priority
@@ -214,7 +229,8 @@ Related object: L<Interchange6::Schema::Result::Product>
 
 belongs_to
   canonical => "Interchange6::Schema::Result::Product",
-  { 'foreign.sku' => 'self.canonical_sku' };
+  { 'foreign.sku' => 'self.canonical_sku' },
+  { join_type => 'left' };
 
 =head2 variants
 
@@ -255,54 +271,6 @@ has_many
   "sku",
   { cascade_copy => 0, cascade_delete => 0 };
 
-=head2 current_price_modifiers
-
-Type: has_many
-
-Related object: L<Interchange6::Schema::Result::PriceModifier>
-
-The following bind values must be included in any search that joins with
-this relationship in the following order:
-
-=over
-
-=item * L<Interchange6::Schema::Result::PriceModifier/end_date>
-
-=item * min value for L<Interchange6::Schema::Result::PriceModifier/quantity>
-
-=item * L<Interchange6::Schema::Result::User/users_id> (can be undef)
-
-=item * L<Interchange6::Schema::Result::PriceModifier/start_date>
-
-=back
-
-See L<Interchange6::Schema::ResultSet::Product/listing> for an example of use.
-
-=cut
-
-has_many
-  current_price_modifiers => "Interchange6::Schema::Result::PriceModifier",
-  sub {
-    my $args = shift;
-
-    my $subquery =
-      $args->{self_resultsource}->schema->resultset('UserRole')
-      ->search( { "roles.users_id" => { '=' => \"?" } }, { alias => 'roles' } )
-      ->get_column('roles_id')->as_query;
-
-    return (
-        {
-            "$args->{foreign_alias}.sku" =>
-              { -ident => "$args->{self_alias}.sku" },
-            "$args->{foreign_alias}.end_date" => [ undef, { '>=', \"?" } ],
-            "$args->{foreign_alias}.quantity" => { '<=',  \"?" },
-            "$args->{foreign_alias}.roles_id" =>
-              [ undef, { -in => $subquery } ],
-            "$args->{foreign_alias}.start_date" => [ undef, { '<=', \"?" } ],
-        },
-    );
-  };
-
 =head2 inventory
 
 Type: might_have
@@ -370,6 +338,14 @@ has_many
   "sku",
   { cascade_copy => 0, cascade_delete => 0 };
 
+=head2 navigation
+
+Type: many_to_many with navigation
+
+=cut
+
+many_to_many navigations => "navigation_products", "navigation";
+
 =head2 orderlines
 
 Type: has_many
@@ -432,6 +408,121 @@ many_to_many _reviews => "_product_reviews", "message";
 
 Attribute methods are provided by the L<Interchange6::Schema::Base::Attribute> class.
 
+=head2 insert
+
+Override inherited method to call L</generate_uri> method in case L</name>
+and L</sku> have been supplied as arguments but L</uri> has not.
+
+=cut
+
+sub insert {
+    my ( $self, @args ) = @_;
+    if ( $self->name && $self->sku && !$self->uri ) {
+        $self->generate_uri;
+    }
+    $self->next::method(@args);
+    return $self;
+}
+
+=head2 update_price_modifiers
+
+Called when L</price> is updated.
+
+=cut
+
+sub update_price_modifiers {
+    my ( $self, $old_value, $new_value ) = @_;
+
+    my $price_modifiers =
+      $self->price_modifiers->search( { discount => { '!=' => undef } } );
+
+    while ( my $result = $price_modifiers->next ) {
+        $result->update(
+            {
+                price => sprintf( "%.2f",
+                    $new_value - ( $new_value * $result->discount / 100 ) )
+            }
+        );
+    }
+}
+
+=head2 generate_uri($attrs)
+
+Called by L</new> if no uri is given as an argument.
+
+The following steps are taken:
+
+=over
+
+1. Join C<< $self->name >> and C<< $self->uri >> with C<-> and stash
+in C<$uri> to allow manipulation via filters
+
+2. Remove leading and trailing spaces and replace remaining spaces and
+C</> with C<->
+
+3. Search for all rows in L<Interchange6::Schema::Result::Setting> where
+C<scope> is C<Product> and C<name> is <generate_uri_filter>
+
+4. For each row found eval C<< $row->value >>
+
+5. Finally set the value of column L</uri> to C<$uri>
+
+=back
+
+Filters stored in L<Interchange6::Schema::Result::Setting> are executed via
+eval and have access to C<$uri> and also the product result held in 
+C<$self>
+
+Examples of filters stored in Setting might be:
+
+    {
+        scope => 'Product',
+        name  => 'generate_uri_filter',
+        value => '$uri =~ s/badstuff/goodstuff/gi',
+    },
+    {
+        scope => 'Product',
+        name  => 'generate_uri_filter',
+        value => '$uri = lc($uri)',
+    },
+
+=cut
+
+sub generate_uri {
+    my $self = shift;
+
+    my $uri = join("-", $self->name, $self->sku);
+
+    # make sure we have clean utf8
+    try {
+        $uri = Encode::decode( 'UTF-8', $uri, Encode::FB_CROAK )
+          unless utf8::is_utf8($uri);
+    }
+    catch {
+        $self->throw_exception(
+            "Product->generate_uri failed to decode UTF-8 text: $_" );
+    };
+
+    $uri =~ s/^\s+//;       # remove leading space
+    $uri =~ s/\s+$//;       # remove trailing space
+    $uri =~ s{[\s/]+}{-}g;  # change space and / to -
+
+    my $filters = $self->result_source->schema->resultset('Setting')->search(
+        {
+            scope => 'Product',
+            name  => 'generate_uri_filter',
+        },
+    );
+
+    while ( my $filter = $filters->next ) {
+        eval $filter->value;
+        $self->throw_exception("Product->generate_uri filter croaked: $@")
+          if $@;
+    }
+
+    $self->uri($uri);
+}
+
 =head2 path
 
 Produces navigation path for this product.
@@ -585,6 +676,11 @@ Returns lowest price from L</price> and L<Pricing price|Interchange6::Schema::Re
 
 Throws exception on bad arguments though unexpected keys in the hash reference will be silently discarded.
 
+If the query was constructed using
+L<Interchange6::Schema::ResultSet::Product/with_lowest_selling_price> then
+the cached value will be used rather than running a new query B<UNLESS>
+arguments are supplied in which case a new query is performed.
+
 =cut
 
 sub selling_price {
@@ -592,7 +688,12 @@ sub selling_price {
 
     my $price = $self->price;
 
-    # if we have args check for hashref and if no args then define hashref
+    if ( $self->has_column_loaded('selling_price') && !defined $args ) {
+
+        # initial query on Product already included selling_price so use it
+
+        return $self->get_column('selling_price');
+    }
 
     if ($args) {
         $self->throw_exception(
@@ -628,10 +729,9 @@ sub selling_price {
 
     # now finally we can see if there is a better price for this customer
 
-    my $dtf = $self->result_source->schema->storage->datetime_parser;
-    my $today = $dtf->format_datetime(DateTime->today);
+    my $today = $self->result_source->schema->format_datetime(DateTime->today);
 
-    my $tier_price = $self->price_modifiers->search(
+    my $selling_price = $self->price_modifiers->search(
         {
             'role.name' => $role_cond,
             quantity => { '<=', $args->{quantity} },
@@ -643,9 +743,52 @@ sub selling_price {
         },
     )->get_column('price')->min;
 
-    $price = defined $tier_price && $tier_price < $price ? $tier_price : $price;
+    return
+      defined $selling_price
+      && $selling_price < $price ? $selling_price : $price;
+}
 
-    return $price;
+=head2 highest_price
+
+If this is a canonical product without variants or a variant product then
+this method will return undef. If highest price is the same as L</selling_price>
+then we again return undef.
+
+If the query was constructed using
+L<Interchange6::Schema::ResultSet::Product/with_highest_price> then
+the cached value will be used rather than running a new query.
+
+This method calls L</variant_count> and L</selling_price> so when constructing
+a resultset query consider also chaining the associated ResultSet methods.
+
+=cut
+
+sub highest_price {
+    my $self = shift;
+
+    return undef unless $self->variant_count;
+
+    my $highest_price;
+
+    if ( $self->has_column_loaded('highest_price') ) {
+        $highest_price = $self->get_column('highest_price');
+    }
+    else {
+        $highest_price = $self->variants->get_column('price')->max;
+    }
+
+    if ( $self->has_column_loaded('selling_price') ) {
+        return $highest_price if $highest_price > $self->selling_price;
+        return undef;
+    }
+
+    if ( $highest_price >
+        $self->self_rs->with_lowest_selling_price->single->selling_price )
+    {
+        return $highest_price;
+    }
+
+    return undef;
 }
 
 =head2 find_variant \%input [\%match_info]
@@ -689,7 +832,9 @@ sub find_variant {
 
         my $variant_attributes = $variant->search_related(
             'product_attributes',
-            {},
+            {
+                'attribute.type' => 'variant',
+            },
             {
                 join     => 'attribute',
                 prefetch => 'attribute',
@@ -753,6 +898,33 @@ Set the 'selected' SKU. For a child product this is set automatically.
 
 =over 4
 
+=item Arguments: C<< cond => $cond >>
+
+=back
+
+Search condition to use. Default is:
+
+    { 'attribute.type' => 'variant' }
+
+=over 4
+
+=item Arguments: C<< order_by => $order_by >>
+
+=back
+
+Ordering to use in query. Default is:
+
+    [
+        { -desc => 'attribute.priority' },
+        { -asc => 'attribute.title' },
+        { -desc => 'attribute_value.priority' },
+        { -asc => 'attribute_value.title' },
+    ]
+
+Set the 'selected' SKU. For a child product this is set automatically.
+
+=over 4
+
 =item Returns: An arrayref of attributes complete with their respective attribute values.
 
 =back
@@ -817,27 +989,47 @@ sub attribute_iterator {
         return $canonical->attribute_iterator(%args);
     }
 
+    my $cond = {
+        'attribute.type' => 'variant',
+    };
+
+    $cond = $args{cond} if defined $args{cond};
+
+    my $order_by = [
+        { -desc => 'attribute.priority' },
+        { -asc => 'attribute.title' },
+        { -desc => 'attribute_value.priority' },
+        { -asc => 'attribute_value.title' },
+    ];
+
+    $order_by = $args{order_by} if defined $args{order_by};
+
     # search for variants
-    my $prod_att_rs = $self->search_related('variants')->search_related(
+    my @prod_atts = $self->search_related('variants')->search_related(
         'product_attributes',
-        {},
+        $cond,
         {
-            join     => 'attribute',
-            prefetch => 'attribute',
-            order_by => 'attribute.priority',
-        },
-    );
+            join    => [
+                'attribute', { product_attribute_values => 'attribute_value' },
+            ],
+            prefetch => [
+                'attribute', { product_attribute_values => 'attribute_value' },
+            ],
+            order_by => $order_by,
+        }
+    )->hri->all;
 
     my %attributes;
-
-    while ( my $prod_att = $prod_att_rs->next ) {
-        my $name = $prod_att->attribute->name;
+    my @ordered_names;
+    foreach my $prod_att ( @prod_atts ) {
+        my $name = $prod_att->{attribute}->{name};
 
         unless ( exists $attributes{$name} ) {
+            push @ordered_names, $name;
             $attributes{$name} = {
                 name             => $name,
-                title            => $prod_att->attribute->title,
-                priority         => $prod_att->attribute->priority,
+                title            => $prod_att->{attribute}->{title},
+                priority         => $prod_att->{attribute}->{priority},
                 value_map        => {},
                 attribute_values => [],
             };
@@ -845,49 +1037,37 @@ sub attribute_iterator {
 
         my $att_record = $attributes{$name};
 
-        my $pav_rs = $prod_att->search_related(
-            'product_attribute_values',
-            {},
-            {
-                join     => 'attribute_value',
-                prefetch => 'attribute_value',
-                order_by => { -desc => 'attribute_value.priority' },
-            }
-        );
-
-        my @values;
-
-        while ( my $prod_att_val = $pav_rs->next ) {
+        foreach my $prod_att_val ( @{ $prod_att->{product_attribute_values} } )
+        {
             my %attr_value = (
-                value    => $prod_att_val->attribute_value->value,
-                title    => $prod_att_val->attribute_value->title,
-                priority => $prod_att_val->attribute_value->priority,
+                value    => $prod_att_val->{attribute_value}->{value},
+                title    => $prod_att_val->{attribute_value}->{title},
+                priority => $prod_att_val->{attribute_value}->{priority},
                 selected => 0,
             );
 
             if ( !exists $att_record->{value_map}->{ $attr_value{value} } ) {
                 $att_record->{value_map}->{ $attr_value{value} } = \%attr_value;
+                push @{$attributes{$name}->{attribute_values}}, \%attr_value;
             }
 
             # determined whether this is the current attribute
-            if ( $args{selected} && $prod_att->sku eq $args{selected} ) {
+            if ( $args{selected} && $prod_att->{sku} eq $args{selected} ) {
                 $att_record->{value_map}->{ $attr_value{value} }->{selected} =
                   1;
             }
         }
     }
 
-    while ( my ( $name, $record ) = each %attributes ) {
-        $record->{attribute_values} =
-          [ sort { $b->{priority} <=> $a->{priority} }
-              values %{ delete $record->{value_map} } ];
+    foreach my $key ( keys %attributes ) {
+        delete $attributes{$key}->{value_map};
     }
 
     if ( $args{hashref} ) {
         return \%attributes;
     }
 
-    return [ sort { $b->{priority} <=> $a->{priority} } values %attributes ];
+    return [ map { $attributes{$_} } @ordered_names ];
 }
 
 =head2 add_variants @variants
@@ -908,6 +1088,21 @@ Example for the hash reference (attributes in the first line):
       name => 'Six Small Yellow Tulips',
       uri => 'six-small-yellow-tulips'}
 
+Since there is a risk that attributes names might clash with Product column
+names (for example L</weight>) an improved syntax exists to prevent such
+problems. This is considered to be the preferred syntax:
+
+    {
+        sku   => 'ROD00014-2-6-mid',
+        uri   => 'fishingrod-weight-2-length-6-flex-mid',
+        price => 355,
+        attributes => [
+            { weight => '2' },
+            { length => '6' },
+            { action => 'mid' },
+        ],
+    }
+
 =cut
 
 sub add_variants {
@@ -924,12 +1119,26 @@ sub add_variants {
             die "SKU missing in input for add_variants.";
         }
 
-        # weed out attribute values
+        if ( defined $var_ref->{attributes} ) {
+
+            # new syntax with explicit attributes
+
+            %attr = %{ delete $var_ref->{attributes} };
+        }
+
+        # weed out attribute values that might be mixed in with columns
+        # as happens with old syntax
+
         while ( my ( $name, $value ) = each %$var_ref ) {
             if ( $self->result_source->has_column($name) ) {
                 $product{$name} = $value;
-                next;
             }
+            else {
+                $attr{$name} = $value;
+            }
+        }
+
+        while ( my ( $name, $value ) = each %attr ) {
 
             my ( $attribute, $attribute_value );
 
@@ -941,10 +1150,7 @@ sub add_variants {
                     }
                 );
 
-                if ( $set->count > 1 ) {
-                    die "Ambigious variant attribute '$name' for SKU $sku";
-                }
-                elsif ( !( $attribute = $set->next ) ) {
+                if ( !( $attribute = $set->next ) ) {
                     die "Missing variant attribute '$name' for SKU $sku";
                 }
 
@@ -956,8 +1162,8 @@ sub add_variants {
                 $attr_map{$name}
                 ->find_related( 'attribute_values', { value => $value } ) )
             {
-                die
-"Missing variant attribute value '$value' for attribute '$name' and SKU $sku";
+                die "Missing variant attribute value '$value'"
+                  . " for attribute '$name' and SKU $sku";
             }
 
             $attr{$name} = $attribute_value;
@@ -981,6 +1187,28 @@ sub add_variants {
     return $self;
 }
 
+=head2 discount_percent
+
+If L</selling_price> is lower than L</price> returns the rounded percentage
+discount or undef.
+
+B<NOTE:> for parent products (products that have variants) this will always
+return undef.
+
+=cut
+
+sub discount_percent {
+    my $self = shift;
+
+    if ( $self->variant_count || $self->selling_price == $self->price ) {
+        return undef;
+    }
+
+    return sprintf( "%.0f",
+        ( $self->price - $self->selling_price ) / $self->price * 100 );
+
+}
+
 =head2 media_by_type
 
 Return a Media resultset with the related media, filtered by type
@@ -1059,10 +1287,42 @@ sub top_reviews {
         { rows => $rows, order_by => { -desc => 'rating' } } );
 }
 
+=head2 variant_count
+
+Returns the number of variants of this product. If the query was constructed
+using L<Interchange6::Schema::ResultSet::Product/with_variant_count> then
+the cached value will be used rather than running a new query.
+
+=cut
+
+sub variant_count {
+    my $self = shift;
+    if ( $self->has_column_loaded('variant_count') ) {
+        return $self->get_column('variant_count');
+    }
+    else {
+        return $self->variants->count;
+    }
+}
+
+=head2 has_variants
+
+Alias for L</variant_count> for backwards-compatibility.
+
+=cut
+
+sub has_variants {
+    return shift->variant_count;
+}
+
 =head2 average_rating
 
 Returns the average rating across all public and approved product reviews or undef if there are no reviews. Optional argument number of decimal places of precision must be a positive integer less than 10 which defaults to 1.
 
+If the query was constructed using
+L<Interchange6::Schema::ResultSet::Product/with_average_rating> then
+the cached value will be used rather than running a new query.
+
 =cut
 
 sub average_rating {
@@ -1071,15 +1331,19 @@ sub average_rating {
 
     $precision = 1 unless ( defined $precision && $precision =~ /^\d$/ );
 
-    my $reviews = $self->reviews( { public => 1, approved => 1 } );
+    if ( $self->has_column_loaded('average_rating') ) {
 
-    # we use database AVG function if it has one
-    try {
-        $avg = $reviews->get_column('rating')->func('AVG');
+        # initial query on Product already included average_rating so use it
+
+        $avg = $self->get_column('average_rating');
+    }
+    else {
+
+        # we need a new query
+
+        $avg = $self->reviews( { public => 1, approved => 1 } )
+          ->get_column('rating')->func('AVG');
     }
-    catch {
-        $avg = $reviews->get_column('rating')->sum / $reviews->count;
-    };
     return defined $avg ? sprintf( "%.*f", $precision, $avg ) : undef;
 }
 
@@ -1150,10 +1414,19 @@ quantity of the product in the inventory. For a product variant the
 quantity returned is for the variant itself whereas for a canonical
 (parent) product the quantity returned is the total for all variants.
 
+If the query was constructed using
+L<Interchange6::Schema::ResultSet::Product/with_quantity_in_stock> then
+the cached value will be used rather than running a new query.
+
 =cut
 
 sub quantity_in_stock {
     my $self = shift;
+
+    # if already loaded by resultset query then return that value
+    return $self->get_column('quantity_in_stock')
+      if $self->has_column_loaded('quantity_in_stock');
+
     my $quantity;
     my $variants = $self->variants;
     if ( $variants->has_rows ) {
