@@ -3,7 +3,6 @@ package Test::Message;
 use DateTime;
 use Test::Deep;
 use Test::Exception;
-use Test::More;
 use Test::Roo::Role;
 
 test 'simple message tests' => sub {
@@ -15,7 +14,7 @@ test 'simple message tests' => sub {
     # fixtures
     $self->message_types;
 
-    my $schema = $self->schema;
+    my $schema = $self->ic6s_schema;
 
     my ( $data, $result );
 
@@ -83,22 +82,22 @@ test 'simple message tests' => sub {
     lives_ok( sub { $result->delete }, "delete message" );
     cmp_ok( $rset_message->count, '==', 0, "We have zero messages" );
 
-    $data->{author} = 333333;
+    $data->{author_users_id} = 333333;
 
     throws_ok( sub { $result = $rset_message->create($data) },
         qr/.*/, "FK error with bad user" );
 
-    delete $data->{author};
+    delete $data->{author_users_id};
 
-    $data->{approved_by} = 333333;
+    $data->{approved_by_users_id} = 333333;
 
     throws_ok( sub { $result = $rset_message->create($data) },
         qr/.*/, "FK error with bad approved_by" );
 
-    delete $data->{approved_by};
+    delete $data->{approved_by_users_id};
 
-    $data->{author}      = $author;
-    $data->{approved_by} = $approver;
+    $data->{author_users_id}      = $author->id;
+    $data->{approved_by_users_id} = $approver->id;
 
     lives_ok( sub { $result = $rset_message->create($data) },
         "add good author and approved_by" );
@@ -106,6 +105,7 @@ test 'simple message tests' => sub {
     cmp_ok( $rset_message->count, '==', 1, "We have one message" );
 
     cmp_ok( $result->author->id, '==', $author->id, "has correct author" );
+    cmp_ok( $result->author->email, 'eq', $author->email, "has correct author" );
 
     cmp_ok( $result->approved_by->id,
         '==', $approver->id, "has correct approver" );
@@ -142,7 +142,7 @@ test 'order comments tests' => sub {
     $self->addresses;
     $self->message_types;
 
-    my $schema = $self->schema;
+    my $schema = $self->ic6s_schema;
 
     my $rset_message = $schema->resultset('Message');
     my $rset_order_comment = $schema->resultset('OrderComment');
@@ -199,9 +199,9 @@ test 'order comments tests' => sub {
     cmp_ok( $schema->resultset('Order')->count, "==", 1, "We have 1 order" );
 
     $data = {
-        title   => "Initial order comment",
-        content => "Please deliver to my neighbour if I am not at home",
-        author  => $user->id,
+        title           => "Initial order comment",
+        content         => "Please deliver to my neighbour if I am not at home",
+        author_users_id => $user->id,
     };
 
     lives_ok( sub { $order->set_comments($data) },
@@ -283,8 +283,8 @@ test 'order comments tests' => sub {
                     title => "Initial order comment",
                     content =>
                       "Please deliver to my neighbour if I am not at home",
-                    author => $user->id,
-                    type   => 'order_comment',
+                    author_users_id => $user->id,
+                    type            => 'order_comment',
                 }
             }
         ],
@@ -349,12 +349,11 @@ test 'product reviews tests' => sub {
 
     my ( $product, $variant, $author, $approver, $rset, $result );
 
-    my $rset_message = $self->schema->resultset('Message');
+    my $rset_message = $self->ic6s_schema->resultset('Message');
 
     lives_ok(
         sub {
-            $product =
-              $self->products->search( { canonical_sku => undef } )->first;
+            $product = $self->products->find( { sku => 'os28066' } );
         },
         "grab canonical product from fixtures"
     );
@@ -379,10 +378,10 @@ test 'product reviews tests' => sub {
         sub {
             $rset_message->create(
                 {
-                    title   => "not a review",
-                    content => "not a review",
-                    author  => $author->id,
-                    type    => "blog_post",
+                    title           => "not a review",
+                    content         => "not a review",
+                    author_users_id => $author->id,
+                    type            => "blog_post",
                 }
             );
         },
@@ -393,9 +392,9 @@ test 'product reviews tests' => sub {
         sub {
             $result = $product->set_reviews(
                 {
-                    title   => "massive bananas",
-                    content => "Love them",
-                    author  => $author->id
+                    title           => "massive bananas",
+                    content         => "Love them",
+                    author_users_id => $author->id
                 },
             );
         },
@@ -407,19 +406,19 @@ test 'product reviews tests' => sub {
     cmp_ok( $product->_reviews->count, '==', 1, "parent has 1 _reviews" );
     cmp_ok( $variant->_reviews->count, '==', 0, "variant has 0 _reviews" );
 
-    cmp_ok( $self->schema->resultset('Message')->count,
+    cmp_ok( $self->ic6s_schema->resultset('Message')->count,
         '==', 2, "2 Message rows" );
 
-    cmp_ok( $self->schema->resultset('ProductReview')->count,
+    cmp_ok( $self->ic6s_schema->resultset('ProductReview')->count,
         '==', 1, "1 ProductReview row" );
 
     lives_ok(
         sub {
             $result = $product->set_reviews(
                 {
-                    title   => "massive bananas",
-                    content => "Love them",
-                    author  => $author->id
+                    title           => "massive bananas",
+                    content         => "Love them",
+                    author_users_id => $author->id
                 },
                 {
                     title   => "cool as ice",
@@ -435,10 +434,10 @@ test 'product reviews tests' => sub {
     cmp_ok( $product->_reviews->count, '==', 2, "parent has 2 _reviews" );
     cmp_ok( $variant->_reviews->count, '==', 0, "variant has 0 _reviews" );
 
-    cmp_ok( $self->schema->resultset('Message')->count,
+    cmp_ok( $self->ic6s_schema->resultset('Message')->count,
         '==', 3, "3 Message rows" );
 
-    cmp_ok( $self->schema->resultset('ProductReview')->count,
+    cmp_ok( $self->ic6s_schema->resultset('ProductReview')->count,
         '==', 2, "2 ProductReview rows" );
 
     lives_ok(
@@ -454,7 +453,7 @@ test 'product reviews tests' => sub {
     cmp_ok( $product->_reviews->count, '==', 3, "parent has 3 _reviews" );
     cmp_ok( $variant->_reviews->count, '==', 0, "variant has 0 _reviews" );
 
-    cmp_ok( $self->schema->resultset('Message')->count,
+    cmp_ok( $self->ic6s_schema->resultset('Message')->count,
         '==', 4, "4 Message rows" );
 
     lives_ok( sub { $rset = $author->reviews }, "grab reviews for author" );
@@ -474,7 +473,7 @@ test 'product reviews tests' => sub {
 
     lives_ok( sub { $product->delete }, "delete parent" );
 
-    cmp_ok( $self->schema->resultset('Message')->count,
+    cmp_ok( $self->ic6s_schema->resultset('Message')->count,
         '==', 1, "1 Message row" );
 
     # cleanup
