@@ -1,7 +1,6 @@
 package Test::Payment;
 
 use Test::Exception;
-use Test::More;
 use Test::Roo::Role;
 
 test 'payment tests' => sub {
@@ -10,7 +9,7 @@ test 'payment tests' => sub {
 
     my $self = shift;
 
-    my $schema = $self->schema;
+    my $schema = $self->ic6s_schema;
 
     my $user;
 
